@@ -4,6 +4,7 @@ import re
 import sys
 import typing
 from datetime import datetime
+from enum import Enum
 
 import pytest
 from django import __version__ as DJANGO_VERSION
@@ -14,8 +15,8 @@ from rest_framework import generics, serializers
 
 from drf_spectacular.openapi import AutoSchema
 from drf_spectacular.plumbing import (
-    analyze_named_regex_pattern, detype_pattern, follow_field_source, force_instance, is_field,
-    is_serializer, resolve_type_hint,
+    analyze_named_regex_pattern, build_basic_type, detype_pattern, follow_field_source,
+    force_instance, is_field, is_serializer, resolve_type_hint,
 )
 from drf_spectacular.validation import validate_schema
 from tests import generate_schema
@@ -90,6 +91,18 @@ class NamedTupleB(typing.NamedTuple):
     b: str
 
 
+class LanguageEnum(str, Enum):
+    EN = 'en'
+    DE = 'de'
+
+
+# Make sure we can deal with plain Enums that are not handled by DRF.
+# The second base class makes this work for DRF.
+class InvalidLanguageEnum(Enum):
+    EN = 'en'
+    DE = 'de'
+
+
 TYPE_HINT_TEST_PARAMS = [
     (
         typing.Optional[int],
@@ -133,9 +146,16 @@ TYPE_HINT_TEST_PARAMS = [
     ), (
         typing.Optional[typing.Union[str, int]],
         {'oneOf': [{'type': 'string'}, {'type': 'integer'}], 'nullable': True}
+    ), (
+        LanguageEnum,
+        {'enum': ['en', 'de'], 'type': 'string'}
+    ), (
+        InvalidLanguageEnum,
+        {'enum': ['en', 'de']}
     )
 ]
 
+
 if DJANGO_VERSION > '3':
     from django.db.models.enums import TextChoices  # only available in Django>3
 
@@ -244,3 +264,8 @@ def test_type_hint_extraction(no_warnings, type_hint, ref_schema):
 def test_analyze_named_regex_pattern(no_warnings, pattern, output):
     re.compile(pattern)  # check validity of regex
     assert analyze_named_regex_pattern(pattern) == output
+
+
+def test_unknown_basic_type(capsys):
+    build_basic_type(object)
+    assert 'could not resolve type for "<class \'object\'>' in capsys.readouterr().err
